//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of CPCtelera: An Amstrad CPC Game Engine
//  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <cpctelera.h>
#include <stdio.h>
#include <string.h>
#include "main.h"
#include "absolutas.h"
#include "enemies.h"
#include "tiles.h"
#include "menu.h"
#include "game.h"
#include "funciones.h"
#include "graphics.h"
#include "render.h"
#include "m_intro.h"

const u8 paletteM1[4] = {HW_BLACK, HW_WHITE, HW_ORANGE, HW_BRIGHT_WHITE}; 
const u8 titles[40][17] = {
//Para apostrofes cuidado con las distintas que hay. Para asegurarse copiar de otro texto ' o ALT+39    
"  Where am I?   ",
"The Walking Dead",
"   Ghost hunt   ",
"Sting Loving you",
"   Bee Quick    ",
"The Spider Nest ",
"      Run!      ",
"    The Lake    ",
"  What's Next?  ",
" The Evil Tree  ",
"  Shark Attack  ",
"Octopus's Garden",
"    The Cave    ",
" Now U C me ... ",
"     Swamps     ",
" Treasure Room  ",
"    Darkness    ",
"    Snakepit    ",
"  Crowded Room  ",
"    The Lair    ",
" Back to Skull  ",
"The Army of Dead",
"   Mine-Slime   ",
" The Lost Order ",
"  Hell's Gate   ",
"Sons of the Dead",
"  Hide & Seek   ",
"   The Garden   ",
"   Cementery    ",
"    The Dead    ",
"  The Batcave   ",
"Valley of Kings ",
" Circle of Fire ",
"   The Coven    ",
"   Jailbreak    ",
"Burn, Burn, Burn",
"   Oh Mummy!    ",
"    The Wall    ",
"  End is Near   ",
"    THE END     "
};

TEntity entities[MAX_ENTITIES]; 
TEnemy* enemies_ptr;
u8 music_counter;
//u8 counter;
u8 time;
u8 music_speed;
//u8 color_black;
u8* mem_start;
u8 mem_page;
u8 swap_memvideo;
u8 max_energy;
u8 screen;
u16 hiscore;
u16 score;
u8 level;
u8 bonus;
u8 powerup;
u8 powerups[12];
u8 powerupsTaken[5];
u8 ptr_powerupsTaken;
u8 target_level;
u8 text_buffer;
TCellBuffer cell_buffer[MAX_CELLS_SCREEN]; 
u8 powerupsOnScreen[4]; 
u8 frame;
u8 dual_stick;
u8 y_powerups;
u8 rooms[41];
u8 enemies_on_screen;
u8 end_screen;
u8 afterIntro;
u8 language;
u8 flipbuffer[MAX_FLIPBUFFER];
u8 touched;
u8 y1;
u8 y2;
u8 y3;
u8 restart;
u8 color_changed;

u8 const txtText_1[8] = "LEVEL ^";
u8 const txtText_2[8] = "ATTACK";
//u8 const txtText_3[8] = "IS DEAD";
u8 const txtText_4[10] = "TRY AGAIN";
u8 const txtText_5[9] = "TAKE TWO";
u8 const txtText_6[9] = "TAKE ONE";
u8 const txtText_7[10] = "BIG SHOTS";
u8 const txtText_8[9] = "CRITICAL";
u8 const txtText_9[11] = "EXTRA LIFE";
u8 const txtText_10[8] = "FORESEE";
u8 const txtText_11[9] = "IMMUNITY";
u8 const txtText_12[9] = "ATTACK ^";
//u8 const txtText_13[10] = "HEALS +10";
u8 const txtText_14[8] = "FREEZE ";
u8 const txtText_15[9] = "LEVEL +1";
u8 const txtText_16[9] = "LOLLIPOP";
u8 const txtText_17[10] = "CRITICAL!";
u8 const txtText_18[8] = "RETURNS";
//u8 const txtText_19[9] = "TO FIGHT";
u8 const txtText_20[9] = "IMMUNITY";
//u8 const txtText_21[10] = "HEALS +10";
u8 const txtText_22[7] = "TIME!!";
u8 const txtText_23[7] = "SCREEN";
u8 const txtText_24[8] = "EXP LVL";
u8 const txtText_25[24] = "It was only a bad dream"; 
u8 const txtText_26[28] = "Too many halloween candies ";
u8 const txtText_27[28] = "??? this is not a happy end";
u8 const txtText_28[29] = "Do I need a special object ?";
u8 const txtText_29[22] = "OTHER KEY: START GAME";
u8 const txtText_30[20] = "Code: @cperezgrin";
u8 const txtText_31[23] = "Graphics: Ricardo Oyon";
u8 const txtText_32[27] = "Music & Fx: Antonio Perez";
u8 const txtText_33[10] = "WARNING!!";


void waitBorder() {
   //Esperar a que el raster haya llegado al borde o al marco del área de juego
   /*for (;;) {
      if (counter == 5)
         break;
   } */ 
   cpct_waitVSYNC();
}

void playFX (u8 instrumento, u8 nota) {
// SFXPlay admite notas de 0 a 143, es decir 144 notas. Siendo 12 notas por octava, daría para 12 octavas
// Sin embargo, Arkos sólo muestra octavas de 0 a A
// Para obtener una nota hay que calcular
// ((octava * 12) + nota (teniendo en cuenta semitonos)) - 1. Por ejemplo E2 => (2 * 12) + 5 - 1

   cpct_akp_SFXInit (m_sfx);
   cpct_akp_SFXPlay (instrumento, 15, nota, 0, 0, AY_CHANNEL_B);
}

void playMusic(){
   //El reproductor Arkos hace uso de los registros auxiliares (AF',BC',...) por lo que si luego se hace uso
   //de funciones que hagan uso de ellos (como easytiles) puede corromperse el programa si llega la interrupción a medio proceso.
   //Por eso, es mejor curarse en salud y guardarlos siempre de manera manual antes de llamar a la reproducción de la música
   __asm
      EXX
      PUSH AF
      PUSH BC
      PUSH DE
      PUSH HL
   __endasm;
   cpct_akp_musicPlay();
   __asm
      POP HL
      POP DE
      POP BC
      POP AF
      EXX
   __endasm;
}

void intHandler(){
   cpct_scanKeyboard_if();  //Para capturar teclado para quitar música o salir durante el juego

   //Las interrupciones se llaman 300 veces por segundo, la música debe sonar 50 veces por segundo (50hz)
   //por lo que sólo hay que llamar al siguiente bloque de música cada 6 veces
   if (!music_counter) {   
      playMusic();

      music_counter = music_speed;
   }

   music_counter--;

/*   if (!counter) {   
      counter = 6;
   }

   counter--;*/
}

void init();

void main(void) {
   //Cambiar el comienzo de la pila y 
   //Borrar Memoria de Vídeo y Buffer
   cpct_setStackLocation ((void*) 0x8000);

   init();
   cpct_memset ((u8*) 0xC000, 0x00, 0x4000);
   
   while (1) {
      resetValues();
      //Menu
      menu();
      //Juego
      game();
   }
   
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Código que sólo es necesario ejecutar al principio (antes de jugar) pues será sobreescrito cuando empiece a usarse 
//el doble buffer de vídeo
// (0x8000 se reserva para gráficos de la intro) llega hasta 0x8D8E
// (0x8D8E se reserva para música de la intro)  llega hasta 0x905B
// (0x9100 se reserva para textos de la intro) llega hasta 0x9C32
// Por eso será escrito a partir de 0x9C32
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CPCT_ABSOLUTE_LOCATION_AREA(0x9C32);
void setInterruptAtVSYNCStart(void *intHdl){
   //Asegurarnos que cambiamos la gestión de interrupciones al terminar VSync
   //pues el primero puede saltar si estamos a mitad de VSync. Por eso se pasan 2 interrupciones más y entra en espera.
   //(halt paraliza la ejecución hasta que se produzca una nueva interrupción)
   //Ahora es seguro que saltará de la espera al llegar el CRTC al VSync
   cpct_waitVSYNC();
   __asm
      halt 
      halt
   __endasm;
   //Nos aseguramos que CRTC estaba a mitad de pantalla
   cpct_waitVSYNC(); //Me aseguro que se espera al final del pintado de la pantalla para cambiar el código de la interrupción
   cpct_setInterruptHandler(intHdl);
}


void texts(){   
   if (language == 1) {
      cpct_memcpy (txtTutorial1_1, txtCastellano, 1093);
      cpct_memcpy (titles, txtTitulos, 680);

      strcpy (txtText_1,"NIVEL ^");
      strcpy (txtText_2,"ATAQUE:");
      //strcpy (txtText_3,"OTRA");
      strcpy (txtText_4,"[OTRA?");
      strcpy (txtText_5,"COGE DOS");
      strcpy (txtText_6,"CODE UNO");
      strcpy (txtText_7,"+DISPARO");
      strcpy (txtText_8,"CRITICO");
      strcpy (txtText_9,"VIDA EXTRA");
      strcpy (txtText_10,"Visi=n");
      strcpy (txtText_11,"INMUNE");
      strcpy (txtText_12,"ATAQUE ^");
      //strcpy (txtText_13,"HP +10");
      strcpy (txtText_14,"CONGELA");
      strcpy (txtText_15,"NIVEL +1");
      strcpy (txtText_16,"PIRULETA");
      strcpy (txtText_17,"]Cr<tico!");
      strcpy (txtText_18,"VUELVE");
      //strcpy (txtText_19,"TO LIFE");
      strcpy (txtText_20,"INMUNE");
      //strcpy (txtText_21,"HEALS +10");
      strcpy (txtText_22,"TIEMPO");
      strcpy (txtText_23,"FASE");
      strcpy (txtText_24,"NIVEL");
      strcpy (txtText_25,"S=lo fue una pesadilla"); 
      strcpy (txtText_26,"demasiados dulces en h'ween");
      strcpy (txtText_27," Esto no parece un final ");
      strcpy (txtText_28,"[Necesito un <tem especial? ");
      strcpy (txtText_29,"OTRA TECLA:  EMPEZAR ");
      strcpy (txtText_30,"C=digo: @cperezgrin");
      strcpy (txtText_31,"Gr|ficos: Ricardo Oy=n");
      strcpy (txtText_32,"M>sica & Fx: Antonio P;rez");
      strcpy (txtText_33,"]CUIDADO!");
   }
}

void intro(){ 
   u8 i,k;
   
   for (k=0; k<5;k++) {
      cpct_drawSolidBox (cpct_getScreenPtr (CPCT_VMEM_START, 8, 0), 0x00,32, 96);   
      cpct_drawSolidBox (cpct_getScreenPtr (CPCT_VMEM_START, 0, 90), 0x00,60, 20);   
      cpct_drawSolidBox (cpct_getScreenPtr (CPCT_VMEM_START,60, 90), 0x00,20, 20);   
      waitBorder();
      cpct_drawSprite (&G_intro_moon, cpct_getScreenPtr (CPCT_VMEM_START, 8 + (2*k),  32 - (8*k)), 16, 64);
      cpct_drawSprite (&G_intro_01[0] + (k*480), cpct_getScreenPtr (CPCT_VMEM_START, 55,  130), 10, 48);
      if (k == 0) {       
         mydrawStringM1(txtIntro1, cpct_getScreenPtr(CPCT_VMEM_START, 37,90));
         mydrawStringM1(txtIntro2, cpct_getScreenPtr(CPCT_VMEM_START, 27,100));
      }
      if (k == 1) {
         mydrawStringM1(txtIntro3, cpct_getScreenPtr(CPCT_VMEM_START,  4,100));
      }
      if (k == 2) {
         mydrawStringM1(txtIntro4, cpct_getScreenPtr(CPCT_VMEM_START, 14,100));
      }
      if (k == 3) {
         mydrawStringM1(txtIntro5, cpct_getScreenPtr(CPCT_VMEM_START, 15,100));
      }
      if (k == 4) {
         mydrawStringM1(txtIntro6, cpct_getScreenPtr(CPCT_VMEM_START, 27,100));
      }
      
      for (i=0; i<4;i++){         
         pause();
      }
   }
   cpct_drawSolidBox (cpct_getScreenPtr (CPCT_VMEM_START, 16, 0), 0x00,16, 64);   
   cpct_drawSprite (&G_logo, cpct_getScreenPtr (CPCT_VMEM_START, 27,   30), 25, 34);

}

void mLoop(u8 step){
   u8 i,j;

   if (step == 7){
      score = 999;
   }

   renderScoreboard(mem_start);
   pause();
   pause();

   resetEntities(0);

   //Posición inicio protagonista y dirección disparo
   entities[0].x = 26;
   entities[0].y = 100;
   entities[1].data1 = -1; //Por defecto, disparar hacia arriba
   entities[1].data2 = 0;
   if (step == 2 || step == 4 || step == 6 || step == 7) {
      entities[2].id = 2;
      entities[2].height = 16;
      entities[2].width = 3;
      entities[2].sprite_f1 = (u8*) &G_zombie_f1[0];
      entities[2].sprite_f2 = (u8*) &G_zombie_f2[0];
      if (step == 4) {
         entities[2].sprite_f1 = (u8*) &G_spider_f1[0];
         entities[2].sprite_f2 = (u8*) &G_spider_f2[0];
      }
      if (step == 6) {
         entities[2].sprite_f1 = (u8*) &G_scorpio_f1[0];
         entities[2].sprite_f2 = (u8*) &G_scorpio_f2[0];
      }
      if (step == 7) {
         entities[2].sprite_f1 = (u8*) &G_skull_f1[0];
         entities[2].sprite_f2 = (u8*) &G_skull_f2[0];
      }
      
      entities[2].movH = 1;
      entities[2].x = 4;
      entities[2].y = 104;
      if (step == 4 || step == 6 || step == 7)
         entities[2].y = 124;
      entities[2].attack = ATTACK_ZOMBI;
      entities[2].energy = ENERGY_ZOMBI;
   }

   end_screen = 0;

   while (end_screen != 2) {
      entities[0].energy = 70;
      waitBorder();
      keyboard(); 
      if (step == 2 || step == 5) {
        entities[0].movV=0; 
        entities[0].movH=0;
      }

      if (step < 3 || step == 5) { //No disparar
         entities[1].x = 0; 
      } else {
         shoot();
      }
      
      iA(); //IA Enemigos
      movementsAllowed(); //Revisar movimientos y colisiones
      collisions();
      waitBorder();
      renderDelete(); 
       //Borrar puertas
       if (!enemies_on_screen){
         end_screen = 2;
         if (step == 4) {
            end_screen = 1;
            enemies_on_screen = 1;
         }
         if (step != 7)
            renderText(mem_start,3,0);
         else
            renderText(mem_start,2,level);

         cpct_drawSolidBox (cpct_getScreenPtr (mem_start, 24, 4), 0x00,8, 16); 
       }

       if (step == 5) {
         if (enemies_on_screen == 120) 
            cpct_drawSolidBox (cpct_getScreenPtr (mem_start, 4, 20),0x00,48,160);  

         if (enemies_on_screen % 20 == 0 && enemies_on_screen) {
            i = (240 - enemies_on_screen) / 20;
            j = i;
            if (enemies_on_screen <= 120)
               j = j - 6;

            cpct_drawSprite (&G_power_up1[0] + (i*64), cpct_getScreenPtr(mem_start, 4, 32 + (j*24)), 4, 16);
            mydrawStringM1(&txtPowerUp1_1[0] + (i*46), cpct_getScreenPtr(mem_start, 8, 32 + (j*24)));  
            mydrawStringM1(&txtPowerUp1_2[0] + (i*46), cpct_getScreenPtr(mem_start, 8, 40 + (j*24)));  
            pause();pause();
         }
       } else {
          renderSprites(); //Pintado-Pintar
      }

      //salió por la puerta (Sólo puede llegar a esa coordenada y cuando fin_nivel > 0)
       if (entities[0].y == 16) {
         end_screen = 2;
       }

      //Salidas programadas
      if (step == 1 && (entities[0].movH || entities[0].movV)) {
         enemies_on_screen--;
      }
      if (step == 2 || step == 3 || step == 5) {
         enemies_on_screen--;
      }
   }
   clearScreen();
}


void tutorial(){
   u8 i;
   u8* text;
   
   mem_start = (u8*) 0xC000;

   for (i=0;i<5;i++) {
      powerupsTaken[i]=255; //Inicialmente ninguno cogido
   }

   cpct_memset ((u8*) 0xC000, 0x00, 0x4000);
   text = (u8*) &txtTutorial1_1[0];
   for (i=1; i<8; i++) {
      resetValues();
      renderFrame(mem_start);
      cpct_drawSolidBox (cpct_getScreenPtr (CPCT_VMEM_START, 0, 180),0xFF       ,56,20);  //recuadro
      cpct_drawSolidBox (cpct_getScreenPtr (CPCT_VMEM_START, 1, 181),0x00,54,18);  //recuadro
      mydrawStringM1(text   , cpct_getScreenPtr(CPCT_VMEM_START, 2, 182));  
      mydrawStringM1(text+27, cpct_getScreenPtr(CPCT_VMEM_START, 2, 190));  
      if (i == 1) {
         entities[0].immunity = 0;
         enemies_on_screen = 45; // A los 30 pasos se saldrá
      }
      if (i == 2 ) {
         entities[0].immunity = RESET_ANIMA;
         enemies_on_screen = 90;
      }
      if (i == 3) {
         entities[0].immunity = 0;
         enemies_on_screen = 120;
      }
      if (i == 5) {
         enemies_on_screen = 240;
      }
      if (i == 4 || i == 6 || i == 7) {
         enemies_on_screen = 1;
      }

      text +=54;
      mLoop(i); 
      pause();
   }
   entities[0].y = 0;  //Para que no aparezca protagonista en attract mode
}

void init(){  
   u8 exit,i;
   u32 semilla;

   // Disable firmware, clear and colour screen (black)
   cpct_disableFirmware();  

   music_speed = 8;
   music_counter = music_speed;
   cpct_akp_musicInit((u8*) m_intro);

   setInterruptAtVSYNCStart(intHandler); 

   language = 0;
   cpct_setBlendMode (CPCT_BLEND_OR);
   cpct_setBorder(HW_BLACK);

   // Seleccionar Idioma
   cpct_setVideoMode(0);
   cpct_setPalette(paletteM0,16);
   
   cpct_drawSprite (G_flag_1, cpctm_screenPtr(CPCT_VMEM_START,37,92), 3, 5);
   cpct_drawSprite (G_flag_2, cpctm_screenPtr(CPCT_VMEM_START,37,103), 3, 5);

   pauseKeyPressed();
   if (cpct_isKeyPressed(Key_2))
      language=1;

   pauseFreeKeyboard();
   
   //Inicializaciones
   cpct_memset ((u8*) 0xC000, 0x00, 0x4000);
   cpct_setVideoMode(1);
   cpct_setPalette(paletteM1,4);
   cpct_setBorder(HW_BLACK);

   afterIntro = 0;
   //color_black = 0x00; //cpct_px2byteM0 (0, 0);
   cpct_setDrawCharM1(3, 0);
      
//   counter = 5;

   entities[0].id = 0;
   entities[0].height = 16;
   entities[0].width = 4;
   entities[0].frozen = 0;
   entities[1].id = 1;
   enemies_ptr = &enemies[0];

   for (i=1;i<41;i++)
      rooms[i] = i;

   texts();
   semilla = 0;
   mydrawStringM1(txtSkip1, cpct_getScreenPtr((void*) 0xC000, 22, 100));   
   exit = 0;
   semilla = 0;

   while (!exit) {
      semilla++;
      cpct_scanKeyboard_f();
      if (cpct_isAnyKeyPressed_f ()) {
         if (!cpct_isKeyPressed(Key_Y) && !cpct_isKeyPressed(Key_S)) {       
            intro();
         }
         exit = 1;
      }
   }

   //Asegurarnos que se ha dejado de pulsar la tecla anterior 
   pauseFreeKeyboard();
   //intro();

   mydrawStringM1(txtSkip2, cpct_getScreenPtr((void*) 0xC000, 20, 100));   
   exit = 0;
   
   while (!exit) {
      cpct_scanKeyboard_f();
      if (cpct_isAnyKeyPressed_f ()) {
         if (!cpct_isKeyPressed(Key_Y) && !cpct_isKeyPressed(Key_S)) {       
            tutorial();
         }
         exit = 1;
      }
   }

   //Asegurarnos que se ha dejado de pulsar la tecla anterior 
   pauseFreeKeyboard();

   if (!semilla)
      semilla = 1;

   cpct_srand8(semilla);
   hiscore = 0;
   afterIntro = 1;

   i = cpct_rand() % 3;

   if (i == 0) {
      y1 = 138;
      y2 = 148;
      y3 = 158;   
   }

   if (i == 1) {
      y1 = 158;
      y2 = 138;
      y3 = 148;   
   }

   if (i == 2) {
      y1 = 148;
      y2 = 158;
      y3 = 138;   
   }


   *(u8*)&paletteM1[2] = HW_BRIGHT_GREEN; //Volver a dejarlo en verde
   cpct_setPalette(paletteM1,4);
   
}

CPCT_RELOCATABLE_AREA();